const WebSocket = require('ws');
const ResManager = require('./resManager');
const watcher = require('chokidar');
const srcPath = 'src/';
const Path = require('path');
const Builder = require('./builder');

module.exports = class {
    constructor() {
        this.port = 5202;
        this.run();
        this.connections = [];
    }

    run() {
        this.server = new WebSocket.Server({ port: this.port });
        this.builder = new Builder();
        this.builder.watch();
        console.log(`Ws server running on port: ${this.port}`);

        watcher.watch([srcPath, `${srcPath}**/*`], {
            ignoreInitial: true
        }).on('addDir', (path) => {
            this.dirAction('add', path);
        }).on('unlinkDir', (path) => {
            this.dirAction('remove', path);
        });

        this.server.on('connection', (conn) => {
            this.sendAction('listResources', ResManager.getTreeResources(), conn);
        });
    }

    sendAction(action, payload, client) {
        if (client) {
            client.send(JSON.stringify({
                action,
                payload
            }));
        }
        else {
            this.server.clients.forEach((client) => {
                client.send(JSON.stringify({
                    action,
                    payload
                }));
            });
        }
    }

    dirAction(action, path) {
        const pathSplit = Path.relative(srcPath, path).split('/');
        if (pathSplit.length === 1) {
            this.sendAction(`${action}Group`, {
                group: pathSplit[0]
            });
        }
        else if (pathSplit.length === 2) {
            this.sendAction(`${action}Resource`, {
                group: pathSplit[0],
                resource: pathSplit[1]
            });
        }
    }
};
